# -*- coding: utf-8 -*-
#   Copyright (C) 2009, 2012-2013 Rocky Bernstein
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Our local modules
from trepan.processor.command import base_subcmd as Mbase_subcmd
from trepan.processor import cmdfns as Mcmdfns


class SetCmdtrace(Mbase_subcmd.DebuggerSetBoolSubcommand):
    """Set echoing lines read from debugger command files"""

    in_list = True
    min_abbrev = len("cmdt")  # Need at least "set cmdt"
    pass

    def run(self, args):
        Mcmdfns.run_set_bool(self, args)
        dbg = self.debugger
        if hasattr(dbg.intf[-1], "verbose"):
            dbg.intf[-1].verbose = dbg.settings[self.name]
            pass
        return

    pass
