"""
Functional tests. In contrast to most unit tesets,
these tests work with a Trepan debugger object and traces or debugs some code with it, testing some particular
aspect of debugging, such as the ability to step, skip or jump to debugged lines, hit breakpoints, etc.
"""
